/**
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 *
 * @file Abstract class for deferred class code.
 */
/**
 * DeferredClass is a generic class to define the pattern of code loaded after the initial code download.
 *
 * @param Type - The type of the Instance object. This type should define how you create an instance
 *   of the deferred class.
 * @param Instance - The interface of the deferred class.
 *
 * Example:
 * class DeferredFoo extends DeferredClass<typeof Foo, Foo> {
 *   public create(arg1: string, arg2: number): Foo {
 *     return new this._classType(arg1, arg2);
 *   }
 *
 *   protected _require(): Promise<Type> {
 *     // code to lazy load e.g. require.ensure(...) or import(...)
 *   }
 * }
 *
 * @internal
 */
export default abstract class DeferredClass<Type, Instance> {
    /**
     * The singleton instance.
     */
    private static _instance;
    /**
     * Resolves after _loadPromise resolves.
     */
    protected readonly _onAfterLoadPromise: Promise<void>;
    /**
     * The type of the Instance object.
     */
    protected _classType: Type;
    /**
     * The promise used for loading the code.
     */
    protected _loadPromise: Promise<Type>;
    /**
     * Resolve callback from _onAfterLoadPromise. Invoked in DeferredClass.load after the code has loaded.
     */
    protected _resolveOnAfterLoadCallbacks: () => void;
    /**
     * Return the singleton for the derived DeferredClass.
     *
     * @param constructor - The type of the class deriving DeferredClass.
     */
    static instance<ChildClass>(): ChildClass;
    /**
     * Instantiate an instance of Type.
     *
     * @param args - A generic argument list; should match the signature of the constructor for Type.
     *
     * @returns - An instance of Type.
     */
    abstract create(...args: any[]): Instance;
    /**
     * @returns Whether or not the code has been loaded.
     */
    readonly isLoaded: boolean;
    /**
     * @returns Whether or not the code should load. Value should not change during the lifecycle of an instance.
     */
    readonly shouldLoad: boolean;
    /**
     * Loads the deferred chunk. After the returned promise is resolved, DeferredClass.create can be used
     * to create an instance of the deferred class.
     *
     * @returns - The Promise used for loading the code. If DeferredClass.shouldLoad has been overriden
     *   and returns false, then a rejected Promise is returned.
     */
    load(): Promise<Type>;
    /**
     * Returns a thenable Promise to be resolved after the code has finished loading. Use this for cases where
     * you do not want to invoke code loading but want to be notified when the code has loaded.
     *
     * @returns - A promise resolved after the code loads.
     */
    onAfterLoad(): Promise<void>;
    /**
     * Singleton instance.
     */
    protected constructor();
    /**
     * Defines how the deferred class is to be loaded asynchronous.
     *
     * @returns - The promise used for loading the code.
     */
    protected abstract _internalLoad(): Promise<Type>;
}
