"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Validate_1 = require("../Validate");
var Text_1 = require("../Text");
var Log_1 = require("../log/Log");
var EventManager = (function () {
    function EventManager() {
        this._listeners = new Map();
        this._raisedEvents = new Map();
    }
    Object.defineProperty(EventManager, "instance", {
        get: function () {
            if (!this._instance) {
                this._instance = new EventManager();
            }
            return this._instance;
        },
        enumerable: true,
        configurable: true
    });
    EventManager.prototype.raiseEvent = function (eventName, eventArgs) {
        Validate_1.default.isNonemptyString(eventName, 'eventName');
        Validate_1.default.isNotNullOrUndefined(eventArgs, 'eventArgs');
        return this._raiseEventInternal(eventName, eventArgs);
    };
    EventManager.prototype.raiseStickyEvent = function (eventName, eventArgs) {
        Validate_1.default.isNonemptyString(eventName, 'eventName');
        Validate_1.default.isNotNullOrUndefined(eventArgs, 'eventArgs');
        this._raisedEvents.set(eventName, eventArgs);
        return this._raiseEventInternal(eventName, eventArgs);
    };
    EventManager.prototype.registerEvent = function (eventName, observer, eventHandler) {
        Validate_1.default.isNonemptyString(eventName, 'eventName');
        Validate_1.default.isNotNullOrUndefined(observer, 'component');
        Validate_1.default.isNotNullOrUndefined(eventHandler, 'eventHandler');
        if (!this._listeners.has(eventName)) {
            this._listeners.set(eventName, []);
        }
        this._listeners.get(eventName).push({ observer: observer, eventHandler: eventHandler });
        if (this._raisedEvents.has(eventName)) {
            eventHandler.apply(observer, this._raisedEvents.get(eventName));
        }
    };
    EventManager.prototype.unregisterEvent = function (eventName, observer, eventHandler) {
        Validate_1.default.isNonemptyString(eventName, 'eventName');
        Validate_1.default.isNotNullOrUndefined(observer, 'component');
        Validate_1.default.isNotNullOrUndefined(eventHandler, 'eventHandler');
        if (!this._listeners.has(eventName)) {
            return;
        }
        var listeners = this._listeners.get(eventName);
        var filteredListeners = listeners.filter(function (el) { return el.observer !== observer || el.eventHandler !== eventHandler; });
        if (filteredListeners.length === listeners.length) {
            var errorMessage = Text_1.default.format(
            'Failed to remove event handler for component "{0}". Event handler was not registered.', observer.componentId);
            Log_1.default.error(EventManager._logSource, new Error(errorMessage));
        }
        this._listeners.set(eventName, filteredListeners);
    };
    EventManager.prototype._raiseEventInternal = function (eventName, eventArgs) {
        var _this = this;
        if (!this._listeners.has(eventName)) {
            return;
        }
        this._listeners.get(eventName).forEach(function (listener) {
            if (listener.observer.isDisposed) {
                _this.unregisterEvent(eventName, listener.observer, listener.eventHandler);
            }
            else {
                try {
                    listener.eventHandler.call(listener.observer, eventArgs);
                }
                catch (e) {
                    var errorMessage = Text_1.default.format(
                    'Failed to execute event handler for component "{0}"', listener.observer.componentId);
                    Log_1.default.error(EventManager._logSource, new Error(errorMessage));
                }
            }
        });
    };
    EventManager._logSource = 'EventManager';
    return EventManager;
}());
exports.default = EventManager;
