/**
 * Common helper functions for working with URLs.  These utilities are intended to be simple,
 * small, and very broadly applicable.
 *
 * @alpha
 */
export default class UrlUtilities {
    /**
     * If the provided URL is relative, resolve it under the provided base URL. If the provided URL is absolute,
     *  return it.
     * Examples:
     *   resolve('/foo/bar.js', 'http://cdn.com/path/')               ---> 'http://cdn.com/path/foo/bar.js'
     *   resolve('http://cdn.com/foo/bar.js', 'http://cdn.com/path/') ---> 'http://cdn.com/foo/bar.js'
     *   resolve('/foo/bar.js', 'path/')                              ---> 'path/foo/bar.js'
     *
     * @param url - the URL to be resolved
     * @param baseUrl - the base URL to use if the URL is relative
     */
    static resolve(url: string, baseUrl: string): string;
    /**
     * Removes any slash characters from the end of the URL.
     * This function assumes that the input is already a valid absolute or server-relative URL.
     * Examples:
     *   removeEndSlash('http://example.com/') ---> 'http://example.com'
     *   removeEndSlash('/example')            ---> '/example'
     *   removeEndSlash('/')                   ---> ''
     * @param url - the URL to be normalized
     */
    static removeEndSlash(url: string): string;
    /**
     * Removes any slash characters from the beginning of the URL.
     * This function assumes that the input is already a valid absolute or server-relative URL.
     * Examples:
     *   removeLeadingSlash('/example/path.js') ---> 'example/path.js'
     *   removeLeadingSlash('example/')         ---> 'example/'
     *   removeLeadingSlash('/')                ---> ''
     * @param url - the URL to be normalized
     */
    static removeLeadingSlash(url: string): string;
    /**
     * Converts a variable to an OData string literal suitable for usage in a REST URL.
     * The returned string will be enclosed in single quotes, and any single quotes
     * will be escaped.
     *
     * Example usage:
     *
     * const url = "/_api/web/GetFolderByServerRelativeUrl("
     *       + UrlUtilities.convertToODataStringLiteral("/SitePages/Alice's%20Page")
     *       + ")/Files";
     *
     *  // Produces this URL:
     *  // "/_api/web/GetFolderByServerRelativeUrl('/SitePages/Alice''s%20Page')/Files"
     *
     * @internalremarks
     * Standard reference:
     * https://tools.oasis-open.org/version-control/
     *   browse/wsvn/odata/trunk/spec/ABNF/odata-abnf-construction-rules.txt
     * SQUOTE-in-string = SQUOTE SQUOTE ; two consecutive single quotes represent
     *   one within a string literal
     */
    static convertToODataStringLiteral(value: string): string;
}
