/**
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 *
 * @file ManifestStore.ts
 */
import { IClientSideComponentManifest } from '@microsoft/sp-module-interfaces';
import { IPreloadedData } from '../IPreloadedData';
import { IObjectWithKey } from './../DeveloperTools/DeveloperToolsFabric';
export interface IManifestStoreEntry extends IObjectWithKey {
    /**
     * The component id.
     */
    id: string;
    /**
     * The component version.
     */
    version: string;
    /**
     * The manifest V2 payload.
     */
    manifest?: IClientSideComponentManifest;
    /**
     * A debug manifest V2 payload, if one has been provided.
     */
    debugManifest?: IClientSideComponentManifest;
}
/**
 * This class maintains a cache of the manifests on the current page.
 */
export default class ManifestStore {
    private static _instance;
    static readonly instance: ManifestStore;
    /**
     * Map from component id to array of manifest store indexes.
     * This allows to store manifests for different versions of the same component.
     *
     * e.g. 'd1d91016-032f-456d-98a4-721247c305e8' -> [ Version(1.0.0), Version(2.0.0) ]
     */
    private _manifestVersions;
    /**
     * Map from manifest store index string to manifest store entry.
     * Manifest store index is generated through logic in utilities/normalizeComponentId (<id>_<version>)
     * e.g. 'd1d91016-032f-456d-98a4-721247c305e8_1.0.0' -> { ManifestStoreEntry }
     */
    private _manifests;
    /**
     * Set of all the component ids that have their manifest pinned.
     * If a component id has its manifest pinned, no other manifest can be added for that component id,
     * regardless of version or debug manifest.
     *
     * e.g. { '1c6c9123-7aac-41f3-a376-3caea41ed83f', '7263c7d0-1d6a-45ec-8d85-d4d1d234171b' }
     */
    private _pinnedManifests;
    /**
     * Register the preloaded manifests on the manifest store.
     *
     * @param preloadedData - The preloaded data
     *
     * @internal
     */
    registerPreloadedManifests(preloadedData: IPreloadedData): void;
    /**
     * Register debug manifests on the manifest store.
     *
     * @param manifests - A dictionary of debug manifests.
     *
     * @internal
     */
    registerDebugManifests(manifests: {
        [id: string]: IClientSideComponentManifest;
    }): void;
    /**
     * Return a manifest by its component id and version.
     *
     * If version is not provided, it will return the only available version. If more than
     * one version is available and version is not provided, returns undefined.
     *
     * Only return a debug manifest is debug is allowed. If a manifest isn't found by
     * the provided id, return undefined.
     *
     * @param id        - The component id of the manifest to retrieve.
     * @param version   - The version of the component of the manifest to retrieve.
     * @param shouldLog - True if failures should be logged. Defaults to true.
     * @returns           The retrieved manifest, or undefined if one could not be found.
     */
    tryGetManifest(id: string, version?: string, shouldLog?: boolean): IClientSideComponentManifest | undefined;
    /**
     * Return a manifest by its component id and version.
     *
     * If version is not provided, it will return the only available version. If more than
     * one version is available and version is not provided, the function throws.
     *
     * Only return a debug manifest is debug is allowed. If a manifest isn't found by
     * the provided id, the function throws.
     *
     * @param id      - The component id of the manifest to retrieve.
     * @param version - The version of the component of the manifest to retrieve.
     * @returns         The retrieved manifest, or undefined if one could not be found.
     */
    getManifest(id: string, version?: string): IClientSideComponentManifest;
    /**
     * Get all registered manifests.
     *
     * @returns The registered manifests.
     */
    getRegisteredManifests(): IClientSideComponentManifest[];
    /**
     * Returns a map of manifest IDs to manifests.
     *
     * @return Map<string, IManifestStoreEntry>: A map of manifest IDs to manifests.
     *
     * @internal
     */
    _getManifestMap(): Map<string, IManifestStoreEntry>;
    /**
     * Loads additional manifests into the manifest store, updating existing manifests.
     *
     * @param manifests - The manifests to load into the store.
     */
    registerManifests(manifests: IClientSideComponentManifest[], overwriteExisting: boolean): void;
    /**
     * Pins the manifest for a specific component id.
     * That means that no other manifest can be added for the specified component id.
     *
     * @remarks
     * This is used by assemblies to ensure that debug manifests are not replacing components already in use.
     *
     * @param id Component id with only one manifest, which will be pinned.
     *
     * @internal
     */
    _pinManifest(componentId: string): void;
    private _isManifestPinned(componentId);
    /**
     * Internal implementation of `getManifest` that toggles whether failures should be logged or not.
     */
    private _getManifest(id, version, shouldLog);
    private _getManifestFromStoreEntry(manifestEntry);
    private _addManifest(manifest, overwriteExisting);
    private _addDebugManifest(manifest);
    private _internalAddManifest(manifest, isDebug, overwriteExisting?);
    /**
     * Adds the manifest to the versions map.
     *
     * If the component id is not present in the map, adds a new entry in the map.
     * If the id and version are already present, it does nothing.
     */
    private _addManifestToVersionsMap(manifest);
    /**
     * Returns the index for a component id and version.
     * The index might not be for the same version, but a compatible one.
     *
     * If an index is not found, returns undefined.
     * If an index is request without a version, and there are multiple versions,
     * returns an error.
     * If more than one compatible version is found, an error is logged and
     * returns the highest compatible version.
     */
    private _getExistingIndex(id, versionString?);
    /**
     * Gets the manifest store index for a component id without version.
     *
     * If no version is found for the id, returns undefined.
     * If there are too many manifests for the component id, logs an error and returns undefined.
     */
    private _getUniqueManifestStoreIndex(id);
    private _createIndexFromManifest(manifest);
    private _createIndex(id, version);
    private _findDebugIndex(id, versions);
}
