import { Guid } from '@microsoft/sp-core-library';
import O365GroupAssociation, { IO365GroupAssociationData } from './O365GroupAssociation';
/**
 * Object that represents the data necessary to build an SPSite object.
 *
 * @internal
 */
export interface ISPSiteData {
    absoluteUrl: string;
    cdnPrefix: string;
    classification: string;
    correlationId: string;
    group?: IO365GroupAssociationData;
    id: string;
    isNoScriptEnabled: boolean;
    recycleBinItemCount: number;
    serverRelativeUrl: string;
    serverRequestPath: string;
    sitePagesEnabled: boolean;
}
/**
  * This class is primarily used with the PageContext class.  It provides contextual
  * information for the SharePoint site collection ("site") that hosts the page.
  *
  * @public
  */
export default class SPSite {
    private _absoluteUrl;
    private _cdnPrefix;
    private _classification;
    private _correlationId;
    private _group;
    private _id;
    private _isNoScriptEnabled;
    private _recycleBinItemCount;
    private _serverRelativeUrl;
    private _serverRequestPath;
    private _sitePagesEnabled;
    /**
     * @internal
     */
    constructor(data: ISPSiteData);
    /**
     * Returns the absolute URL for this SPSite.
     *
     * @remarks
     * Example: "https://example.com/sites/PubSite"
     */
    readonly absoluteUrl: string;
    /**
     * Returns the prefix of the application's specified cdn or an empty string if there isn't one.
     */
    readonly cdnPrefix: string;
    /**
     * Returns the classification of the site.
     */
    readonly classification: string;
    /**
     * Returns the correlation id to the current server request.
     */
    readonly correlationId: Guid;
    /**
     * Contextual information about the O365 Group associated with this site.
     * If there is no O365Group associated with the current site, this property will be undefined.
     */
    readonly group: O365GroupAssociation | undefined;
    /**
     * The GUID that identifies the SPSite on the server.
     */
    readonly id: Guid;
    /**
     * Returns true if isNoScript has been enabled on the SPSite.
     */
    readonly isNoScriptEnabled: boolean;
    /**
     * The amount of items in the recyle bin.
     */
    readonly recycleBinItemCount: number;
    /**
     * Returns the server-relative URL for this SPSite.
     *
     * @remarks
     * Example: "/sites/PubSite"
     */
    readonly serverRelativeUrl: string;
    /**
     * Returns serverRelativeUrl of the original request.
     *
     * @remarks
     * Example: "/teams/SPClientTest/SitePages/Home.aspx"
     */
    readonly serverRequestPath: string;
    /**
     * Returns true if SitePages are enabled on this SPSite.
     */
    readonly sitePagesEnabled: boolean;
    /**
     * Function that throws an exception if an ISPSiteData object or its parameters are
     * null or undefined.
     *
     * @param data - Data used to construct an ISPSiteData object
     */
    private _validate(data);
}
