import SPTimeZone, { ISPTimeZoneData } from './SPTimeZone';
/**
 * Object that represents the data necessary to build an SPUser object.
 *
 * @internal
 */
export interface ISPUserData {
    displayName: string;
    email: string;
    isAnonymousGuestUser: boolean;
    isExternalGuestUser: boolean;
    loginName: string;
    preferUserTimeZone: boolean;
    timeZoneInfo?: ISPTimeZoneData;
}
/**
  * This class is primarily used with the PageContext class.  It provides contextual information
  * for the SharePoint user that is accessing the page.
  *
  * @public
  */
export default class SPUser {
    private _displayName;
    private _email;
    private _isAnonymousGuestUser;
    private _isExternalGuestUser;
    private _loginName;
    private _preferUserTimeZone;
    private _timeZone;
    /**
     * @internal
     */
    constructor(data: ISPUserData);
    /**
     * The display name for the current user.
     *
     * @remarks
     * Example: "John Doe"
     */
    readonly displayName: string;
    /**
     * The email address for the current user.
     *
     * @remarks
     * Example: "example@contoso.com"
     */
    readonly email: string;
    /**
     * Returns if the current user is an anonymous guest.
     */
    readonly isAnonymousGuestUser: boolean;
    /**
     * Returns true if the current user is an external guest.
     */
    readonly isExternalGuestUser: boolean;
    /**
     * The login name for current user.
     *
     * @remarks
     * Example: "i:0#.w|domain\user"
     */
    readonly loginName: string;
    /**
     * This boolean represents if a the user or web's time zone settings should be used
     * to display the current time.
     */
    readonly preferUserTimeZone: boolean;
    /**
     * Returns the user's regional timezone settings or undefined if they haven't been set.
     *
     * @alpha
     */
    readonly timeZoneInfo: SPTimeZone | undefined;
    /**
     * Function that throws an exception if an ISPUserData object or its parameters are
     * null or undefined.
     *
     * @param data - Data used to construct an ISPUserData object
     */
    private _validate(data);
}
