import { Guid } from '@microsoft/sp-core-library';
import { IODataBasePermission } from '@microsoft/sp-odata-types';
import SPPermission from './SPPermission';
import SPTimeZone, { ISPTimeZoneData } from './SPTimeZone';
/**
 * Object that represents the data necessary to build an SPWeb object.
 * @internal
 */
export interface ISPWebData {
    absoluteUrl: string;
    id: string;
    isAppWeb: boolean;
    language: number;
    logoUrl: string;
    permissions: IODataBasePermission;
    timeZoneInfo?: ISPTimeZoneData;
    serverRelativeUrl: string;
    templateName: string;
    title: string;
    description: string;
}
/**
  * This class is primarily used with the PageContext class. It provides contextual
  * information for the SharePoint site ("web") that hosts the page.
  *
  * @public
  */
export default class SPWeb {
    private _absoluteUrl;
    private _id;
    private _isAppWeb;
    private _language;
    private _logoUrl;
    private _permissions;
    private _serverRelativeUrl;
    private _templateName;
    private _timeZone;
    private _title;
    private _description;
    /**
     * @internal
     */
    constructor(data: ISPWebData);
    /**
     * Returns the absolute URL for this SPWeb.
     * Example: "https://example.com/sites/PubSite/SubWeb"
     */
    readonly absoluteUrl: string;
    /**
     * The GUID that identifies the SPWeb on the server.
     */
    readonly id: Guid;
    /**
     * Returns true if this SPWeb the container web for an SPApp.
     */
    readonly isAppWeb: boolean;
    /**
     * Returns the locale identifier (LCID) for the default language of the website.
     *
     * @remarks
     * Example: 1033 represents the locale identifier for en-US.
     */
    readonly language: number;
    /**
     * Returns the absolute URL of the website logo.
     *
     * @remarks
     * Example: https://example.com/sites/PubSite/SubWeb/logo.jpg
     */
    readonly logoUrl: string;
    /**
     * Return the SPPermission object that represents the set of permissions that the
     * current user has for interacting with the web.
     */
    readonly permissions: SPPermission;
    /**
     * Returns the web's regional timezone settings or undefined if they haven't been set.
     *
     * @alpha
     */
    readonly timeZoneInfo: SPTimeZone | undefined;
    /**
     * Returns the server-relative URL for this SPWeb.
     *
     * @remarks
     * Example: "/sites/PubSite/SubWeb"
     */
    readonly serverRelativeUrl: string;
    /**
     * Returns the string representing the numeric identifier for the site definition or
     * site template that was used to create the site.
     *
     * @remarks
     * Example: "1" represents the team site template when creating a new site on SharePoint.
     */
    readonly templateName: string;
    /**
     * Returns the title of the current SPWeb.
     */
    readonly title: string;
    /**
     * Returns the description of the current SPWeb.
     */
    readonly description: string;
    /**
     * Function that throws an exception if an ISPWebData object or its parameters are
     * null or undefined.
     *
     * @param data - Data used to construct an ISPWebData object
     */
    private _validate(data);
}
