/**
 * A singleton class to manage web part _Animations. The class triggers pre-configured reveal animation on web parts
 * when they come in view in a modern page. It uses ScrollReveal lib to accomplish this, which is defer loaded when
 * _Animations.init() is called. Use _Animations.reveal() to register the web part container elements; this
 * method can be used before or after calling init. For best outcome call the init() as soon after PLT/loading glass.
 *
 * VSO #476309 - Rationalize/Refactor scroll animations
 *
 * @internal
 */
export default class _Animations {
    private static _instance;
    private _scrollRevealLibrary;
    private _scrollReveal;
    private _scrollableParent;
    private _pendingElements;
    /**
    * Get singleton instance of _Animations class
    */
    static readonly instance: _Animations;
    /**
     * Initiates loading of deferred scrollReveal lib, and when done calls to configure and reveal on elements which are
     * pending. To be called right after PLT/glass has been loaded.
     */
    init(): void;
    /**
     * Registers elements for scroll reveal animation. If scrollReveal lib is not yet configured we track the elements by
     * adding it to a pending list, but if it’s configured we then call scrollReveal.reveal().
     *
     * @param elem - The container element on which we want scroll reveal animation. In our case, container of a web part
     */
    reveal(elem: HTMLElement): void;
    /**
     * Sets the scrollable parent of elements on which scroll reveal is to be called, and then calls for configure and
     * reveal on elements which are pending.
     *
     * @param elem - The scrollable parent of elements (in our case a web part) on which we want animation.
     */
    setScrollableParent(elem: HTMLElement): void;
    /**
    * Clears the current scrollable parent and the configuration
    */
    clear(): void;
    /**
    * Private constructor of singleton class
    */
    private constructor();
    private _reveal(elem);
    private _tryConfigureAndRevealPending();
    private _ensureScrollRevealLoaded();
}
