/**
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 *
 * @file Interface that a client side web part host needs to implement.
 */
import { ServiceScope } from '@microsoft/sp-core-library';
import { IWebPartConfigurationEventCallback } from '../propertyPane/propertyPane/PropertyPaneDefinitions';
/**
 * A web part host is a component, control or a page that hosts client side web parts.
 * The web part infrastructure provides many features inbuilt e.g. an HttpClient to make
 * server requests and the SPPageContext. The porpose of this interface is not to replicate
 * those feature. This interface outlines the services and features where there needs to be
 * a direct interaction between the host and the web part infrastructure. This interface also
 * outlines some services where the web part host may want to override the default
 * implementations provided by the infrastructure. Let us discuss some examples:
 *
 *  - APIs like setDirty, webPartConfigurationEventCallback help the web part infrastucture
 *    communicate the fact that some web part is in a dirty state  or web part communication
 *    events to the host.
 *
 * - In future we expect that this interface will be used to provide services where the host
 *   might want to override the default implemenations. e.g.
 *   - the web part status rendering service. The default implementation of this service
 *     provides an office fabric spinny. But a certain host could easily want to show a
 *     different spinny.
 *
 *   - The event aggregation service. Currently the web part manager hosts a single event
 *     event aggregator. But there are possibilities when a page can have multiple hosts
 *     and each host may want to scope its event aggregation service.
 *
 * @alpha
 */
interface IWebPartHost {
    /**
     * The current ServiceScope.
     */
    readonly serviceScope: ServiceScope;
    /**
     * API for the webpart to mark the containing host as dirty.
     */
    setDirty?: (instanceId: string, data?: any) => void;
    /**
     * Web part configuration event callback.
     */
    webPartConfigurationEventCallback?: IWebPartConfigurationEventCallback;
    /**
     * Optional flag to disable lazy loading of the web parts. By default it is enabled.
     */
    isViewportLoadingDisabled?: boolean;
}
export default IWebPartHost;
