"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var sp_core_library_1 = require("@microsoft/sp-core-library");
var lodash = require("@microsoft/sp-lodash-subset");
var office_ui_fabric_react_1 = require("office-ui-fabric-react");
var ScrollEventHandler = (function () {
    function ScrollEventHandler() {
        this._eventListeners = new Set();
        this._isListeningWindow = false;
        this._lazyHandler = lodash.throttle(this._scrollEventHandler, 10).bind(this);
    }
    ScrollEventHandler._findScrollableParent = function (startingElement) {
        var DATA_IS_SCROLLABLE_ATTRIBUTE = 'data-is-scrollable';
        var el = startingElement;
        var body = document.body;
        while (el && el !== body) {
            if (el.getAttribute(DATA_IS_SCROLLABLE_ATTRIBUTE) === 'true') {
                return el;
            }
            el = el.parentElement;
        }
        el = startingElement;
        while (el && el !== body) {
            var styles = getComputedStyle(el);
            if (styles) {
                var overflowY = styles.getPropertyValue('overflow-y');
                if (overflowY && (overflowY === this.SCROLL || overflowY === 'auto')) {
                    return el;
                }
            }
            el = el.parentElement;
        }
        if (!el || el === body) {
            el = undefined;
        }
        return el;
    };
    Object.defineProperty(ScrollEventHandler, "instance", {
        get: function () {
            if (ScrollEventHandler._instance === undefined) {
                ScrollEventHandler._instance = new ScrollEventHandler();
            }
            return ScrollEventHandler._instance;
        },
        enumerable: true,
        configurable: true
    });
    ScrollEventHandler.prototype.reset = function () {
        this._scrollableParents = [];
        this._eventListeners.clear();
        this._cachedClientRect = undefined;
    };
    ScrollEventHandler.prototype.register = function (element, callbackFunction, scrollableParent) {
        var _this = this;
        if (!this._scrollableParents || !this._scrollableParents.length || !this._optimizeScrollableParent) {
            this._scrollableParents =
                scrollableParent && this._optimizeScrollableParent
                    ? [scrollableParent]
                    : this._findScrollableParents(element);
        }
        this._scrollableParents.forEach(function (parent, index, allParents) {
            var totalRegisteredCount = _this._updateRegisteredChildren(parent, 1);
            if (totalRegisteredCount === 1) {
                _this._listenEvents(parent);
            }
        });
        this._listenEvents(window);
        this._eventListeners.add(callbackFunction);
    };
    ScrollEventHandler.prototype.unregister = function (element, callbackFunction) {
        var _this = this;
        sp_core_library_1.Validate.isNotNullOrUndefined(callbackFunction, 'unregister callback function');
        if (!this._eventListeners.has(callbackFunction)) {
            return;
        }
        this._eventListeners.delete(callbackFunction);
        var parents = this._optimizeScrollableParent
            && this._scrollableParents && this._scrollableParents.length
            ? this._scrollableParents
            : this._findScrollableParents(element);
        parents.forEach(function (parent, index, allParents) {
            var count = _this._updateRegisteredChildren(parent, -1);
            if (count <= 0) {
                _this._unlistenEvents(parent);
            }
        });
        if (this._eventListeners.size === 0) {
            this._unlistenEvents(window);
        }
    };
    ScrollEventHandler.prototype._scrollEventHandler = function (event) {
        var _this = this;
        if (!this._cachedClientRect || this._cacheExpired) {
            this._cachedClientRect = this._scrollableParents[this._scrollableParents.length - 1].getBoundingClientRect();
            this._cachedClientRectAge = Date.now();
        }
        this._eventListeners.forEach(function (listener) {
            listener(event, _this._cachedClientRect);
        });
    };
    ScrollEventHandler.prototype._findScrollableParents = function (startingElement) {
        var parents = [];
        var parent = startingElement;
        while (parent) {
            parent = ScrollEventHandler._findScrollableParent(parent);
            if (parent) {
                parents.push(parent);
                parent = parent.parentElement;
            }
        }
        return parents.length ? parents : [document.body];
    };
    ScrollEventHandler.prototype._listenEvents = function (parent) {
        if (!parent) {
            return;
        }
        var isWindow = parent === window;
        if (!isWindow || (isWindow && !this._isListeningWindow)) {
            this._addEventListerer(parent);
            if (isWindow) {
                this._isListeningWindow = true;
            }
        }
    };
    ScrollEventHandler.prototype._addEventListerer = function (element) {
        element.addEventListener(ScrollEventHandler.SCROLL, this._lazyHandler, false);
        element.addEventListener(ScrollEventHandler.RESIZE, this._lazyHandler, false);
    };
    ScrollEventHandler.prototype._unlistenEvents = function (parent) {
        if (!parent) {
            return;
        }
        var isWindow = parent === window;
        if (!isWindow || (isWindow && this._isListeningWindow)) {
            this._removeEventListener(parent);
            if (isWindow) {
                this._isListeningWindow = false;
            }
        }
    };
    ScrollEventHandler.prototype._removeEventListener = function (element) {
        element.removeEventListener(ScrollEventHandler.SCROLL, this._lazyHandler);
        element.removeEventListener(ScrollEventHandler.RESIZE, this._lazyHandler);
    };
    ScrollEventHandler.prototype._updateRegisteredChildren = function (scrollableParent, count) {
        var lazyChilds = count + Number(scrollableParent.getAttribute(ScrollEventHandler._eventTrackerAttribute));
        if (count !== 0) {
            scrollableParent.setAttribute(ScrollEventHandler._eventTrackerAttribute, lazyChilds.toString());
        }
        return lazyChilds;
    };
    Object.defineProperty(ScrollEventHandler.prototype, "_optimizeScrollableParent", {
        get: function () {
            return !sp_core_library_1._SPKillSwitch.isActivated(sp_core_library_1.Guid.parse('5b7e10fc-9e1f-46ae-b9f7-a4925c48b72a'), '8/28/2017', 'Do not optimize calculating scrollable parent of web part');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ScrollEventHandler.prototype, "_cacheExpired", {
        get: function () {
            var optimize = !sp_core_library_1._SPKillSwitch.isActivated(sp_core_library_1.Guid.parse('09c8a198-874b-4d5f-ad7e-bddb3b4916f8'), '9/7/2017', 'Do not Cache ClientRect value');
            return optimize
                ? (this._cachedClientRectAge - Date.now() >= ScrollEventHandler.CACHE_EXPIRED)
                : true;
        },
        enumerable: true,
        configurable: true
    });
    ScrollEventHandler._eventTrackerAttribute = 'data-sp-webpart-event-listener';
    ScrollEventHandler.SCROLL = 'scroll';
    ScrollEventHandler.RESIZE = 'resize';
    ScrollEventHandler.CACHE_EXPIRED = 100;
    tslib_1.__decorate([
        office_ui_fabric_react_1.autobind
    ], ScrollEventHandler.prototype, "register", null);
    tslib_1.__decorate([
        office_ui_fabric_react_1.autobind
    ], ScrollEventHandler.prototype, "_findScrollableParents", null);
    tslib_1.__decorate([
        office_ui_fabric_react_1.autobind
    ], ScrollEventHandler.prototype, "_unlistenEvents", null);
    tslib_1.__decorate([
        office_ui_fabric_react_1.autobind
    ], ScrollEventHandler.prototype, "_updateRegisteredChildren", null);
    tslib_1.__decorate([
        office_ui_fabric_react_1.autobind
    ], ScrollEventHandler, "_findScrollableParent", null);
    return ScrollEventHandler;
}());
exports.default = ScrollEventHandler;
