/// <reference types="react" />
/**
 * @Copyright (c) Microsoft Corporation.  All rights reserved.
 *
 * @file Root component of the inbuilt webpart property configuration user experience.
 *
 */
import * as React from 'react';
import { IPropertyPaneData } from './IPropertyPane';
/**
 * Propertypane component props.
 */
export interface IPropertyPaneProps {
    data: IPropertyPaneData;
}
/**
 * Propertypane component state.
 */
export interface IPropertyPaneState {
    isSaveButtonEnabled: boolean;
    saveConfirmationMessage: string;
    currentPage: number;
}
/**
 * Page direction.
 */
export declare enum PageDirection {
    back = -1,
    next = 1,
}
/**
 * A map to maintain the state(collapsed or not) of all the groups of a PropertyPane page.
 * This dictionary is updated when the user makes any changes to the PropertyPane group.
 */
export declare type PropertyPaneGroupsStateMap = {
    [key: string]: boolean;
};
/**
 * A map to maintain the PropertyPaneGroupsState for the PropertyPane pages.
 */
export declare type PropertyPanePagesStateMap = {
    [key: string]: PropertyPaneGroupsStateMap;
};
/**
 * Propertypane component.
 *
 * @internal
 */
export default class PropertyPane extends React.Component<IPropertyPaneProps, IPropertyPaneState> {
    refs: {
        [key: string]: Element;
        propertyPane: HTMLElement;
    };
    /**
     * properties bag of the web part.
     */
    private _properties;
    private _invalidEntries;
    private _cssClassForActionableElement;
    private _propertyPane;
    private _propertyPaneContent;
    private _currentlyConfiguredWebPartId;
    private _isPropertyPaneReplaced;
    /**
     * Dictionary to maintain the PropertyPaneGroupsState for each PropertyPane page.
     *
     * @private
     * @type PropertyPanePagesStateMap
     * @memberOf PropertyPane
     */
    private _propertyPanePagesStateMap;
    /**
     * Constructor to instantiate the propertypane component.
     */
    constructor(props: IPropertyPaneProps);
    /**
     * React component lifecycle event.
     * @param props - new props for the component.
     */
    componentWillReceiveProps(props: IPropertyPaneProps): void;
    componentDidMount(): void;
    componentDidUpdate(props: IPropertyPaneProps): void;
    /**
     * React component lifecycle method.
     * Added this method to avoid, unnecessary renders.
     */
    shouldComponentUpdate(nextProps: IPropertyPaneProps, nextState: IPropertyPaneState): boolean;
    render(): React.ReactElement<{}>;
    /**
     * Renders the description for no configuration scenario.
     */
    private _renderNoConfigurationState();
    private readonly _descriptionId;
    /**
     * Renders page on the property pane.
     */
    private _renderPage();
    private _setContainerDiv(containerDiv);
    /**
     * Private method which gets called when user manually collapses or opens the propertyPane group,
     * when the Propertypane page's displayGroupsAsAccordion prop is true.
     *
     * @param pageNumber - page on which the group's state changed.
     * @param groupName - name of the group.
     * @param isCollapsed - indicates whether the group is collapsed or not.
     */
    private _propertyPaneGroupStateChanged(pageNumber, groupName, isCollapsed);
    /**
     * Returns if the empty page should be rendered in property pane.
     */
    private readonly _shouldRenderEmptyPage;
    /**
     * Flips the page based on the direction.
     * @param direction - direction of the page flip. Positive value is forward direction, negative is backwards.
     */
    private _flipPage(direction);
    private _setEntryState(targetProperty, isValidEntry);
    private _enableSaveButton(enabled);
    private _onChange(targetProperty, value);
    private _handleSaveClick();
    private _handleCloseClick();
    private _raiseEvent(targetProperty, value);
    private _onPropertyPaneKeyDown(ev);
    /**
     * Handles the case when the user presses 'Alt + P'(it's the key combination to toggle the PropertyPane focus)
     * while the Propertypane is in focus.
     *
     * @private
     */
    private _propertyPaneShouldLooseFocus();
    private _setTimeOutForFocus();
    private readonly _isManagingFocusBySettings;
    private _focusOnFirstActionableElement(propertyPane);
    /**
     * Method to update the tab indexes of all the elements in the root to the given value.
     * @param tabIndex
     */
    private _updateTabIndexes(tabIndex, ignoreTabIndexes);
    /**
     * Resetting the PropertyPane state.
     * @param webPartId - web part id to be configured with.
     */
    private _resetState(webPartId);
    private _getCssBottomValueForPageContentElement();
}
