import { PropertyPaneState, IWebPartConfigurationEventCallback } from '../propertyPane/PropertyPaneDefinitions';
import { IClientSideWebPartInternal } from '../../core/IClientSideWebPartInternal';
/**
 * Type definition of the web part getter delegate.
 *
 * @internal
 */
export interface IWebPartGetter {
    (id: string): IClientSideWebPartInternal;
}
export default class PropertyPaneController {
    /**
     * PropertyPane animation timeout (in milliseconds).
     */
    private static ANIMATION_TIMEOUT;
    /**
     * PropertyPane configuration complete timeout for Reactive webparts (in milliseconds).
     */
    private static CONFIGURATION_COMPLETE_TIMEOUT;
    /**
     * PropertyPane container id.
     */
    private static PROPERTY_PANE_CONTAINER_ID;
    /**
     * PropertyPane container id.
     */
    private static PROPERTY_PANE_CONTAINER_NOFLEXBOX_ID;
    private static flexLayoutKillSwitch;
    private _propertyPaneContainerId;
    /**
     * PropertyPane data of the web part currently being configured.
     */
    private _currentPropertyPaneData;
    private _logSource;
    /**
     * Indicating whether the PropertyPane is open or not.
     */
    private _isOpen;
    /**
     * Id of the web part being configured.
     */
    private _currentlyConfiguredWebPartId;
    /**
     * If true, the property pane was rendered by a web part and not by Canvas or any other source.
     */
    private _isRenderedByWebPart;
    /**
     * Page content DOM element, which is the sibling of the PropertyPane container element. If available, this element
     * will be animated in and out as the PropertyPane opens and closes.
     */
    private _pageContentElement;
    /**
     * PropertyPane container DOM element on the host page.
     */
    private _propertyPaneContainer;
    /**
     * The root DOM element (div) of the PropertyPane component.
     */
    private _propertyPaneElement;
    /**
     * Method to get a web part instance.
     */
    private _webPartGetter;
    /**
     * Configuration completion timeout
     */
    private _configurationCompletionTimeout;
    /**
     * PropertyPane lifecycle event callback (e.g. OpenComplete, CloseComplete, etc...).
     */
    private _configurationEventCallback;
    /**
     * @todo: This should be removed once the task (SPPPLAT VSO#222337) is completed.
     */
    private _workbenchCommandBarElement;
    /**
     * Instantiates the PropertyPaneController object.
     *
     * @param webPartGetter - delegate to get the web part.
     * @param callback - callback to be called when a configuration event happens.
     */
    constructor(webPartGetter: IWebPartGetter, callback?: IWebPartConfigurationEventCallback);
    /**
     * Get the currently configured WebPart instance Id
     */
    readonly currentlyConfiguredWebPartId: string | undefined;
    /**
     * Set the property pane state which helps in configuring the web part. The inbuilt PropertyPane is used for the
     * configuration, with the below conditions:
     *
     *   - propertyPaneState is Open and then open the PropertyPane.
     *   - propertyPaneState is Close and then close the PropertyPane.
     *   - propertyPaneState is Toggle and the PropertyPane is closed: in this case we open the PropertyPane and
     *       start the configuration process. This should happen on web part configure button being clicked.
     *   - propertyPaneState is Toggle and the PropertyPane is opened: in this case, if the web part id
     *       is that of a different web part than being currently configured, we keep the PropertyPane open
     *       and change the web part being configured.
     *   - propertyPaneState is Default and the PropertyPane is closed: do nothing
     *   - propertyPaneState is Default and the PropertyPane is opened: change the PropertyPane to display the
     *       settings of the newly selected web part. This should happen when the user navigates between web parts.
     *
     * @param id - instance id of the web part.
     * @param propertyPaneState - indicates in what state the PropertyPane should be.
     * @param renderedByWebPart - is the property pane rendered by a web part and not by Canvas or any other source.
     */
    setPropertyPaneState(id: string, propertyPaneState?: PropertyPaneState, renderedByWebPart?: boolean): void;
    /**
     * Returns true if the property pane rendered by a web part and not by Canvas or any other source.
     */
    isRenderedByWebPart(): boolean;
    isPropertyPaneOpen(): boolean;
    /**
     * Make the PropertyPane disappear when the web part being configured is deleted from the page.
     *
     * @param id - Instance id of the web part.
     */
    onWebPartDelete(id: string): void;
    /**
     * Internal setPropertyPaneState.
     * @param wp - Web part to be configured.
     * @param id - Instance id of the web part to be configured.
     * @param propertyPaneState - State of the PropertyPane.
     */
    private _setPropertyPaneState(wp, id, propertyPaneState?);
    /**
     * A private method to show the property pane.
     *
     * This method will also set the value of the '_isOpen' value appropriately.
     */
    private _showPropertyPane();
    /**
     * A private method to hide the property pane.
     *
     * This method will also set the value of the '_isOpen' value appropriately.
     */
    private _hidePropertyPane();
    /**
     * Method to render the PropertyPane.
     *
     * @param currentWebpart - Current web part for which the PropertyPane is to be displayed.
     * @param shouldRecreatePropertyPaneElement - Indicating whether PropertyPane should be recreated or not.
     * @param resetPropertyPaneState - Indicating whether PropertyPane state should be reset or not.
     */
    private _renderPropertyPane(currentWebpart, shouldRecreatePropertyPaneElement?, resetPropertyPaneState?);
    /**
     * Method to load the property pane component into the propertypanecontainer element using the
     * currentPropertyPaneData.
     *
     * @private
     * @param shouldRecreatePropertyPaneElement - Indicating whether PropertyPane should be recreated or not.
     *
     * @memberOf PropertyPaneController
     */
    private _loadPropertyPaneComponent(shouldRecreatePropertyPaneElement);
    /**
     * Allow the PropertyPane to re-render itself. Used when navigating pages.
     */
    private _render();
    /**
     * PropertyPane field change event handler.
     *
     * @param propertyName - Name of the PropertyPane field changed.
     * @param newValue - New value.
     *  This value could be undefined/empty in the case of custom field.
     */
    private _onPropertyPaneFieldChanged(propertyName, newValue);
    /**
     * Callback to handle the configuration events that originate in the PropertyPane.
     * @example: PropertyPaneClosed, PropertyPaneApplyClicked etc.,
     */
    private _onConfigurationEvent(configurationEvent);
    /**
     * Method to fire the configuration events post toggling the PropertyPane.
     * If toggling opened the PropertyPane then fire both the 'PropertyPaneOpened' and 'PropertyPaneConfigurationStart'
     * events after the ANIMATION_TIMEOUT is elapsed.
     * Else if the toggling resulted in closing the PropertyPane then fire 'PropertyPaneConfigurationComplete' event
     * and then after ANIMATION_TIMEOUT is elapsed fire 'PropertyPaneClosed' event.
     *
     * @private
     */
    private _firePostToggleConfigurationEvents();
    /**
     * Method to fire the configuration events when the PropertyPane did not toggle.
     * Fire the 'PropertyPaneConfigurationComplete' event on the current web part and
     * then 'PropertyPaneConfigurationStart' for the new web part.
     *
     * @private
     * @param id - Id of the web part to be configured.
     */
    private _fireNoToggleConfigurationEvents(id);
    /**
     * Method to fire the configuration event to the host and the base web part.
     *
     * @param { webpartId } - Id of the web part to which the event needs to be sent.
     * @param { configurationEvent } - Kind of configuration event to fire.
     */
    private _fireConfigurationEvent(webpartId, configurationEvent);
    /**
     * Private method to resolve the elements, which are used in slide in slide out effect.
     */
    private _resolvePropertyPaneElements();
    /**
     * Removes all the children of the PropertyPane container.
     */
    private _clearPropertyPaneContainer();
    /**
     * Reset configuration completion timeout.
     */
    private _resetConfigurationCompleteTimeout();
    /**
     * Clear configuration completion timeout.
     */
    private _clearConfigurationCompleteTimeout();
    /**
     * Method to check whether the currently configured PropertyPane is reactive or not.
     */
    private _isCurrentlyConfiguredPropertyPaneReactive();
    /**
     * Returns the PropertyPane data which has only 'onClose' defined. This is used when we have to display
     * empty property pane, when there is nothing to configure.
     */
    private _getEmptyPropertyPaneData();
    private readonly _isFlexLayoutEnabled;
}
